#!/bin/env python3

import click
import logging
import os
import subprocess
import tempfile
import struct
import sys
import hashlib
import esptool

from .idrc_esp_tools.gen_esp32part import PartitionTable


logger = logging.getLogger('configurer')
logger.setLevel(logging.INFO)

BASE_DIR = os.path.dirname(__file__)
TMP_DIR = os.path.join(os.environ['HOME'], '.hwtester')

NVS_TARGET_NAMESPACE = 'ESP-MDF'
exit_code = 0


class SerialConfigurator:

    def __init__(self, usb_port, baudrate):
        self.usb_port = usb_port
        self.baudrate = baudrate

    def generate_nvs(self, nvs_items=None):

        nvs_items = nvs_items or []

        os.makedirs(TMP_DIR, exist_ok=True)

        with tempfile.NamedTemporaryFile(dir=TMP_DIR, suffix='.bin') as f:

            # dump partition table from a running node
            esptool.main([
                '-p',
                self.usb_port,
                '--after',
                'no_reset',
                'read_flash',
                '0x8000',
                '0x1000',
                f.name,
            ])
            logger.info('Dumped partition table')

            f.seek(0)
            partition_table = PartitionTable.from_binary(f.read())
            nvs_partition = partition_table.find_by_name('nvs')
            assert nvs_partition, 'NVS partition not found!'
            logger.info('NVS partition @%s len: %s', nvs_partition.offset, nvs_partition.size)

        with tempfile.NamedTemporaryFile(dir=TMP_DIR, mode='wt', suffix='.csv') as csv_out:

            # generate CSV
            csv_out.write('key,type,encoding,value\n')
            csv_out.write(f'{NVS_TARGET_NAMESPACE},namespace,,\n')
            for item in nvs_items:
                csv_out.write(f'{item["key"]},data,{item["encoding"]},{item["value"]}\n')
            csv_out.flush()

            with tempfile.NamedTemporaryFile(dir=TMP_DIR, suffix='.bin') as nvs_partition_out:

                # generate partition
                p = subprocess.run([
                    'python3',
                    os.path.join(BASE_DIR, 'idrc_esp_tools/nvs_partition_gen.py'),
                    'generate',
                    csv_out.name,
                    nvs_partition_out.name,
                    str(nvs_partition.size),
                ], capture_output=True, text=True)
                if p.stderr:
                    logger.warning(p.stderr)
                p.check_returncode()
                logger.info('Generated new NVS partition')

                # flash NVS partition
                esptool.main([
                    '-p',
                    self.usb_port,
                    'write_flash',
                    str(nvs_partition.offset),
                    nvs_partition_out.name,
                ])
                logger.info('Flashed new NVS partition')

        logger.info('Ready!')


@click.command()
@click.option('-p', '--port', type=str, required=True, help='USB port to open serial to.')
@click.option('-b', '--baudrate', type=int, default=115200, help='Port baudrate.')
@click.option('--router-ssid', type=str, required=True, help='')
@click.option('--router-password', type=str, required=True, help='')
@click.option('--router-bssid', type=str, default='0' * 12, help='')
@click.option('--mesh-id', type=str, required=True, help='')
@click.option('--mesh-password', type=str, required=True, help='')
@click.option('--mesh-type', type=str, default='idle', help='')
@click.option('--channel', type=int, default=0, help='')
@click.option('--channel-switch-disable/--no-channel-switch-disable', default=False)
@click.option('--router-switch-disable/--no-router-switch-disable', default=False)
@click.option('--mqtt-uri', type=str, required=True, help='')
def main(
    port,
    baudrate,
    router_ssid,
    router_password,
    router_bssid,
    mesh_id,
    mesh_password,
    mesh_type,
    channel,
    channel_switch_disable,
    router_switch_disable,
    mqtt_uri,
):
    if len(router_ssid) > 32:
        raise click.BadParameter('router_ssid must be shorter than 32 characters!')
    if len(router_password) > 64:
        raise click.BadParameter('router_password must be shorter than 64 characters!')
    if len(mesh_password) > 64:
        raise click.BadParameter('mesh_password must be shorter than 64 characters!')
    try:
        _router_bssid = int(router_bssid, 16)
    except Exception:
        raise click.BadParameter('router_bssid is invalid')
    try:
        _mesh_id = int(mesh_id, 16)
    except Exception:
        raise click.BadParameter('mesh_id is invalid')

    if len(mesh_password) == 63:
        encoded_mesh_password = mesh_password
    else:
        ap_sha256 = list(hashlib.sha256(router_password.encode()).digest())
        mesh_sha256 = list(hashlib.sha256(mesh_password.encode()).digest())
        encoded_mesh_password = ['0'] * 64
        for i in range(32):
            mesh_sha256[i] ^= ap_sha256[i]
            tmp = ((mesh_sha256[i] & 0xf0) >> 4) + ord(b'0')
            encoded_mesh_password[2 * i] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
            tmp = (mesh_sha256[i] & 0x0f) + ord(b'0')
            encoded_mesh_password[2 * i + 1] = chr(tmp + ord(b'A') - ord(b'9') if tmp > ord(b'9') else tmp)
        encoded_mesh_password = ''.join(encoded_mesh_password)[:63]

    print(f'''
port = {port}
baudrate = {baudrate}
router_ssid = {router_ssid}
router_password = {router_password}
router_bssid = {router_bssid}
mesh_id = {mesh_id}
mesh_password = {mesh_password}
encoded_mesh_password = {encoded_mesh_password}
mesh_type = {mesh_type}
channel = {channel}
channel_switch_disable = {channel_switch_disable}
router_switch_disable = {router_switch_disable}
mqtt_uri = {mqtt_uri}
    ''')

    '''
    typedef enum {
        MWIFI_MESH_IDLE,
        MWIFI_MESH_ROOT,
        MWIFI_MESH_NODE,
        MWIFI_MESH_LEAF,
        MWIFI_MESH_STA,
    } node_type_t;

    typedef uint8_t mwifi_node_type_t;

    typedef struct {
        char router_ssid[32];
        char router_password[64];
        uint8_t router_bssid[6];
        uint8_t mesh_id[6];
        char mesh_password[64];
        mwifi_node_type_t mesh_type;
        uint8_t channel;
        uint8_t channel_switch_disable;
        uint8_t router_switch_disable;
    } mwifi_config_t;

    typedef struct {
        mwifi_config_t config;
        uint8_t custom[32 + 96];
        uint16_t whitelist_size;
        mconfig_whitelist_t whitelist_data[0];
    } __attribute__((packed)) mconfig_data_t;
    '''

    data = struct.pack(
        '32s' # char router_ssid[32];
        '64s' # char router_password[64];
        '6s' # uint8_t router_bssid[6];
        '6s' # uint8_t mesh_id[6];
        '64s' # char mesh_password[64];
        'B' # mwifi_node_type_t mesh_type;
        'B' # uint8_t channel;
        'B' # uint8_t channel_switch_disable;
        'B' # uint8_t router_switch_disable;
        '128s' # uint8_t custom[32 + 96];
        'H' # uint16_t whitelist_size;
        '', # mconfig_whitelist_t whitelist_data[0]; - NOT SUPPORTED
        router_ssid.encode(), # char router_ssid[32];
        router_password.encode(), # char router_password[64];
        _router_bssid.to_bytes(6, 'big'), # uint8_t router_bssid[6];
        _mesh_id.to_bytes(6, 'big'), # uint8_t mesh_id[6];
        mesh_password.encode(), # char mesh_password[64];
        {
            'idle': 0,
            'root': 1,
            'node': 2,
            'leaf': 3,
            'sta': 4,
        }.get(mesh_type), # mwifi_node_type_t mesh_type;
        channel, # uint8_t channel;
        int(channel_switch_disable), # uint8_t channel_switch_disable;
        int(router_switch_disable), # uint8_t router_switch_disable;
        mqtt_uri.encode(), # uint8_t custom[32 + 96];
        0, # uint16_t whitelist_size;
    )

    runner = SerialConfigurator(usb_port=port, baudrate=baudrate)
    runner.generate_nvs([
        {
            'key': 'mconfig_data',
            'encoding': 'hex2bin',
            'value': data.hex(),
        }
    ])


def _main():
    handler = logging.StreamHandler()
    formatter = logging.Formatter('%(levelname)-10s %(name)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    main()


def run_configurer():
    os.makedirs(TMP_DIR, exist_ok=True)
    _main()


if __name__ == '__main__':
    run_configurer()



#!/bin/env python3

import asyncio
import click
import logging
import os
import sys

from asciimatics.widgets import (
    Frame,
    ListBox,
    Layout,
    Divider,
    VerticalDivider,
    Text,
    Button,
    TextBox,
    Widget,
    Label,
)
from asciimatics.widgets.utilities import THEMES
from asciimatics.scene import Scene
from asciimatics.screen import Screen
from asciimatics.event import KeyboardEvent
from asciimatics.exceptions import StopApplication
from collections import defaultdict
from functools import partial

# idrc_esp_tools modules
from .idrc_esp_tools import serial_asyncio, at
from .tester import SerialTestRunner


logger = logging.getLogger("modbuser")
logger.setLevel(logging.INFO)


THEMES["monochrome2"] = defaultdict(
    lambda: (Screen.COLOUR_WHITE, Screen.A_NORMAL, Screen.COLOUR_BLACK),
    {
        "invalid": (Screen.COLOUR_BLACK, Screen.A_NORMAL, Screen.COLOUR_RED),
        "label": (Screen.COLOUR_WHITE, Screen.A_BOLD, Screen.COLOUR_BLACK),
        "title": (Screen.COLOUR_WHITE, Screen.A_BOLD, Screen.COLOUR_BLACK),
        "selected_field": (Screen.COLOUR_YELLOW, Screen.A_BOLD, Screen.COLOUR_BLACK),
        "selected_focus_field": (
            Screen.COLOUR_YELLOW,
            Screen.A_BOLD,
            Screen.COLOUR_BLUE,
        ),
        "focus_button": (Screen.COLOUR_YELLOW, Screen.A_BOLD, Screen.COLOUR_BLUE),
        "selected_focus_control": (
            Screen.COLOUR_YELLOW,
            Screen.A_BOLD,
            Screen.COLOUR_BLUE,
        ),
        "disabled": (Screen.COLOUR_BLACK, Screen.A_BOLD, Screen.COLOUR_BLACK),
        "edit_text": (Screen.COLOUR_YELLOW, Screen.A_NORMAL, Screen.COLOUR_BLACK),
        "focus_edit_text": (Screen.COLOUR_YELLOW, Screen.A_BOLD, Screen.COLOUR_BLACK),
    },
)


class ModbusView(Frame):
    """
    Main view.
    """

    def __init__(self, screen, queue):
        super(ModbusView, self).__init__(
            screen,
            screen.height,
            screen.width,
            hover_focus=True,
            can_scroll=False,
            title="Modbus Tester v.0.0.1",
        )
        self.testing_queue = queue

        layout = Layout([10, 1, 81], fill_frame=True)
        self.add_layout(layout)

        self._parity_view = ListBox(
            4,
            [
                ("even", 1),
                ("odd", 2),
                ("none", 3),
            ],
            name="parity",
            add_scroll_bar=True,
            on_change=self._on_pick,
            on_select=self._go,
        )
        layout.add_widget(Label("Parity"), 0)
        layout.add_widget(Divider(), 0)
        layout.add_widget(self._parity_view, 0)

        self._function_code_view = ListBox(
            5,
            [
                ("discrete", 1),
                ("coil", 2),
                ("input", 3),
                ("holding", 4),
            ],
            name="function_code",
            add_scroll_bar=True,
            on_change=self._on_pick,
            on_select=self._go,
        )
        layout.add_widget(Label("Function"), 0)
        layout.add_widget(Divider(), 0)
        layout.add_widget(self._function_code_view, 0)

        self._word_order_view = ListBox(
            3,
            [
                ("big", 1),
                ("little", 2),
            ],
            name="word_order",
            add_scroll_bar=True,
            on_change=self._on_pick,
            on_select=self._go,
        )
        layout.add_widget(Label("Word order"), 0)
        layout.add_widget(Divider(), 0)
        layout.add_widget(self._word_order_view, 0)

        self._byte_order_view = ListBox(
            3,
            [
                ("big", 1),
                ("little", 2),
            ],
            name="byte_order",
            add_scroll_bar=True,
            on_change=self._on_pick,
            on_select=self._go,
        )
        layout.add_widget(Label("Byte order"), 0)
        layout.add_widget(Divider(), 0)
        layout.add_widget(self._byte_order_view, 0)

        self._encoding_view = ListBox(
            Widget.FILL_FRAME,
            [
                ("bool", 1),
                ("float", 2),
                ("double", 3),
                ("int8", 4),
                ("int16", 5),
                ("int32", 6),
                ("int64", 7),
                ("uint8", 8),
                ("uint16", 9),
                ("uint32", 10),
                ("uint64", 11),
            ],
            name="encoding",
            add_scroll_bar=True,
            on_change=self._on_pick,
            on_select=self._go,
        )
        layout.add_widget(Label("Encoding"), 0)
        layout.add_widget(Divider(), 0)
        layout.add_widget(self._encoding_view, 0)

        layout.add_widget(VerticalDivider(), 1)

        self.data["baudrate"] = "9600"
        self.data["slave"] = "1"
        self.data["register"] = "10000"
        self.data["output"] = ["Output will appear here..."]
        layout.add_widget(Text("Baudrate:", "baudrate"), 2)
        layout.add_widget(Divider(), 2)
        layout.add_widget(Text("Slave ID:", "slave"), 2)
        layout.add_widget(Divider(), 2)
        layout.add_widget(Text("Register:", "register"), 2)
        layout.add_widget(Divider(), 2)
        self.output = TextBox(
            Widget.FILL_FRAME,
            "Output:",
            "output",
            readonly=True,
            line_wrap=True,
        )
        layout.add_widget(self.output, 2)

        divider = Layout([1])
        self.add_layout(divider)
        divider.add_widget(Divider())

        layout2 = Layout([1, 1, 1, 1])
        self.add_layout(layout2)

        layout2.add_widget(Button("Test", self._go), 0)
        layout2.add_widget(Button("Quit", self._quit), 3)

        self.set_theme("monochrome2")
        self.fix()

        self._parity_view.value = 3
        self._function_code_view.value = 3
        self._word_order_view.value = 2
        self._byte_order_view.value = 2
        self._encoding_view.value = 10

        self._on_pick()

    def _on_pick(self):
        pass

    def _go(self):
        self.save()
        self.testing_queue.put_nowait(
            (
                # //! slave address: raw value
                int(self.data["slave"]),
                # //! baud rate: raw value (ex: 1200 or 2400 or 4800 or 9600 or 14400 or ...)
                int(self.data["baudrate"]),
                # //! parity: [1 = even][2 = odd][3 = none] (cf. protocol)
                int(self.data["parity"]),
                # //! function code: [1 = discret input][2 = coil][3 = input register][4 =
                # //! holding register] (cf protocol)
                int(self.data["function_code"]),
                # //! register address: raw value
                int(self.data["register"]),
                # //! word/byte order: [1 = big endian][2 = little endian] (cf protocol)
                int(self.data["word_order"]),
                int(self.data["byte_order"]),
                # //! encoding: [1 = bool][2 = float][3 = double][4 = int8][5 = int16][6 =
                # //! int32][7 = int64][8 = uint8][9 = uint16][10 = uint32][11 = uint64] (cf
                # //! protocol)
                int(self.data["encoding"]),
            )
        )

    @staticmethod
    def _quit():
        sys.exit(0)


class ModbuserRunner(SerialTestRunner):
    async def modbusize(self, queue, begin_testing_event, modbus_view):
        self.reader, self.writer = await serial_asyncio.open_serial_connection(
            url=self.usb_port,
            baudrate=self.baudrate,
        )
        self.at = at.AT(
            r=self.reader,
            w=self.writer,
            default_timeout=5,
            log=os.path.join(os.path.abspath(os.getcwd()), 'modbus_test.log'),
        )
        try:
            await self.at.TEST.expect_reply(timeout=10)
        except Exception as e:
            logger.error("Hardware did not reply with +TEST acknowledgement")
            logger.error(e)
            sys.exit(1)
        begin_testing_event.set()
        while to_test := await queue.get():
            modbus_view.output.value.append(
                "> AT+TESTMODBUS=" + ",".join(map(str, to_test))
            )
            modbus_view.output.reset()
            modbus_view.screen.force_update(True)
            modbus_view.screen.draw_next_frame()
            await self.at.TESTMODBUS.set(*to_test)
            try:
                reply = await self.at.TESTMODBUS.expect_reply(timeout=5)
                modbus_view.output.value.append("< " + reply.strip())
            except Exception as e:
                modbus_view.output.value.append("< ERROR: " + str(e))
            modbus_view.output.reset()
            modbus_view.screen.force_update(True)
            modbus_view.screen.draw_next_frame()


def global_shortcuts(event):
    if isinstance(event, KeyboardEvent):
        c = event.key_code
        # Stop on Ctrl+Q
        if c in [Screen.ctrl("Q"), Screen.ctrl("q")]:
            raise StopApplication("User terminated app")


async def screen_loop(screen, port, baudrate):
    testing_queue = asyncio.Queue()
    modbus_view = ModbusView(screen, testing_queue)
    screen.set_scenes(
        [
            Scene([modbus_view], -1, name="Main"),
        ],
        unhandled_input=global_shortcuts,
    )
    runner = ModbuserRunner(usb_port=port, baudrate=baudrate)
    runner.set_nvs_testing_flag()
    begin_testing_event = asyncio.Event()
    asyncio.create_task(
        runner.modbusize(testing_queue, begin_testing_event, modbus_view)
    )
    await begin_testing_event.wait()
    while True:
        modbus_view.output.reset()
        screen.draw_next_frame()
        await asyncio.sleep(0.02)


def modbusizer_ui(port, baudrate, screen):
    asyncio.run(screen_loop(screen, port, baudrate))


@click.command()
@click.option("-p", "--port", type=str, help="USB port to open serial to.")
@click.option("-b", "--baudrate", type=int, default=115200, help="Port baudrate.")
def main(port, baudrate):
    Screen.wrapper(partial(modbusizer_ui, port, baudrate))


def run_modbuser():
    main()


if __name__ == "__main__":
    run_modbuser()



import os

from typing import Optional
from fastapi import FastAPI, Request
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from pydantic import BaseModel
from pydantic import BaseSettings

from .tester import run_tester


here = os.path.abspath(os.path.dirname(__file__))
app = FastAPI()
app.mount('/static', StaticFiles(directory=os.path.join(here, 'static')), name='static')
templates = Jinja2Templates(directory=os.path.join(here, 'templates'))


class Settings(BaseSettings):

    tht_tty: Optional[str]
    tht_ui_host: str = '0.0.0.0'
    tht_ui_port: int = 80

    class Config:
        env_prefix = 'idrc_'


settings = Settings()


class ThtArgs(BaseModel):

    tht_tty: Optional[str]


@app.get('/tht/', response_class=HTMLResponse)
async def tht(request: Request):
    return templates.TemplateResponse('tht.html', {'request': request})


@app.post('/tht/api/run/')
async def tht_test(request: Request, args: ThtArgs):
    run_tester()
    return {'result': 'ok'}


def main():
    import uvicorn
    uvicorn.run(
        'teramesh_hardware_tester.server:app',
        host=settings.tht_ui_host,
        port=settings.tht_ui_port,
        reload=True)


if __name__ == '__main__':
    main()
